﻿using DAL;
using Microsoft.Extensions.Configuration;
using Shared.DTO;
using Shared.DTO.Integracion_DGA;
using Shared.DTO.VariablesEntorno;
using Shared.Helper;
using System.Text.Json;

namespace BLL.Integracion_DGA
{
    public class ObtencionDatosDga
    {
        private readonly IConfiguration _configuration;
        private readonly ApiService _apiService;
        private readonly JobsDgaRepository _jobs;
        private readonly LogEnvioRepository _logEnvioRepository;

        public ObtencionDatosDga(IConfiguration configuration, ApiService apiService, JobsDgaRepository jobs, LogEnvioRepository logEnvioRepository)
        {
            _configuration = configuration;
            _apiService = apiService;
            _jobs = jobs;
            FileLoggerHelper.ConfigureLogger(_configuration);
            _logEnvioRepository = logEnvioRepository;
        }

        public async Task ObtenerDatosDga()
        {
            await _logEnvioRepository.InsertarLogProcesoAsync(new LogProceso
            {
                FechaEjecucion = DateTime.UtcNow,
                NombreProceso = "OBTENCION DATOS DGA"
            });

            DateTimeOffset dateEnd = DateTimeOffset.Now;
            DateTimeOffset dateStart = dateEnd.AddHours(-1);

            JsonSerializerOptions options = new JsonSerializerOptions() { PropertyNameCaseInsensitive = true };

            ConsoleLoggerHelper.WriteLineAndLogInfo($"INICIO OBTENER DATOS DGA");
            try
            {
                string apiUrlBase = NexusApiUrl.ApiUrl;
                Dictionary<string, string> headers = new Dictionary<string, string>
                  {
                      { "nexustoken", NexusApiUrl.ApiKey },
                      { "nexusapiversion", NexusApiUrl.Version },
                      { "accept", "application/json" }
                  };

                ConsoleLoggerHelper.WriteLineAndLogInfo($"OBTENIENDO DOCUMENTOS");
                string apiUrlDocuments = apiUrlBase + "/api/Documents";

                // Utiliza el servicio para realizar la solicitud a la API con URL y encabezados personalizados
                string responseData = await _apiService.GetApiResponseAsync(apiUrlDocuments, headers);

                var documento = JsonSerializer.Deserialize<List<DocumentResponse>>(responseData, options);

                if (documento == null || documento.Count == 0) 
                {
                    return;
                }

                foreach (DocumentResponse item in documento)
                {
                    ConsoleLoggerHelper.WriteLineAndLogInfo($"OBTENIENDO TAGVIEWS");
                    string apiUrlTagViews = $"{apiUrlBase}/api/Documents/tagviews/{item.uid}";
                    responseData = await _apiService.GetApiResponseAsync(apiUrlTagViews, headers);
                    TagviewsResponse tagviews = JsonSerializer.Deserialize<TagviewsResponse>(responseData, options) ?? new TagviewsResponse();

                    List<string> listTagsID = tagviews.Columns.Select(x => x.Uid).ToList();

                    HistoricRequest historicRequest = new HistoricRequest
                    {
                        DataSource = NexusApiUrl.DataSource,
                        Resolution = NexusApiUrl.Resolution,
                        Uids = listTagsID,
                        StartTs = dateStart.ToUnixTimeSeconds(),
                        EndTs = dateEnd.ToUnixTimeSeconds()
                    };

                    ConsoleLoggerHelper.WriteLineAndLogInfo($"OBTENIENDO TAGVIEWS HISTORIC");
                    string apiUrlHistoric = $"{apiUrlBase}/api/Documents/tagviews/{item.uid}/historic";
                    responseData = await _apiService.PostApiResponseAsync(apiUrlHistoric, headers, historicRequest);
                    List<HistoricResponse> historicResponse = JsonSerializer.Deserialize<List<HistoricResponse>>(responseData, options) ?? new List<HistoricResponse>();

                    TimeZoneInfo zonaHorariaChile = TimeZoneInfo.FindSystemTimeZoneById("Pacific SA Standard Time");

                    if (item.name == "API - DGA - CAUDALES")
                    {
                        ConsoleLoggerHelper.WriteLineAndLogInfo($"OBTENIENDO DATOS DGA CAUDAL");
                        var caudalData = tagviews.Columns
                            .SelectMany(tag =>
                                historicResponse.Where(h => h.Uid == tag.Uid)
                                    .Select(h => new DGAInsert
                                    {
                                        TAG = tag.Name,
                                        VALOR = h.Value?.ToString(),
                                        FECHAMEDICION = TimeZoneInfo.ConvertTimeFromUtc(
                                            new DateTime(1970, 1, 1).AddSeconds(h.TimeStamp).ToUniversalTime(),
                                            zonaHorariaChile)
                                    })
                            ).ToList();

                        await _jobs.InsertarDgaCaudal(caudalData);
                    }
                    else if (item.name == "API - DGA - NIVELES")
                    {
                        ConsoleLoggerHelper.WriteLineAndLogInfo($"OBTENIENDO DATOS DGA NIVEL");
                        var nivelData = tagviews.Columns
                            .SelectMany(tag =>
                                historicResponse.Where(h => h.Uid == tag.Uid)
                                    .Select(h => new DGAInsert
                                    {
                                        TAG = tag.Name,
                                        VALOR = h.Value?.ToString(),
                                        FECHAMEDICION = TimeZoneInfo.ConvertTimeFromUtc(
                                            new DateTime(1970, 1, 1).AddSeconds(h.TimeStamp).ToUniversalTime(),
                                            zonaHorariaChile)
                                    })
                            ).ToList();

                        await _jobs.InsertarDgaNivel(nivelData);
                    }
                }

                ConsoleLoggerHelper.WriteLineAndLogInfo($"INICIO REGISTRO DE MEDICIONES DGA");
                await _jobs.SpRegistrarMedicionesDga();
                ConsoleLoggerHelper.WriteLineAndLogInfo($"FIN REGISTRO DE MEDICIONES DGA");
            }
            catch (Exception ex)
            {
                FileLoggerHelper.LogError($"{ex.Message}", ex);
                ConsoleLoggerHelper.WriteLineAndLogInfo($"{ex.Message}", ConsoleColor.Red);
            }
        }
    }
}
